__all__ = [
    'DurationUnit',
    'UserRestriction',
    'AllProjectsUserRestriction',
    'PoolUserRestriction',
    'ProjectUserRestriction',
    'SystemUserRestriction',
]
import datetime
import enum
import toloka.client.primitives.base
import toloka.util._extendable_enum
import typing


class DurationUnit(enum.Enum):
    """An enumeration.
    """

    MINUTES = 'MINUTES'
    HOURS = 'HOURS'
    DAYS = 'DAYS'
    PERMANENT = 'PERMANENT'


class UserRestriction(toloka.client.primitives.base.BaseTolokaObject):
    """A base class for access restrictions.

    Toloker's access to projects or pools can be restricted.
    You can set the duration of the ban or apply an unlimited restriction.

    Use the [set_user_restriction](toloka.client.TolokaClient.set_user_restriction.md) method to apply a restriction
    and the [delete_user_restriction](toloka.client.TolokaClient.delete_user_restriction.md) method to remove it.

    Attributes:
        id: The ID of the restriction.
        user_id: The ID of the Toloker.
        private_comment: A comment visible to the requester only.
        will_expire: The UTC date and time when the access will be restored by Toloka.
            If the parameter isn't set, then the restriction is active until you remove it calling the `delete_user_restriction` method.
        created: The UTC date and time when the restriction was applied. Read-only field.

    Example:
        Restricting access to a project and removing the restriction.

        >>> new_restriction = toloka_client.set_user_restriction(
        >>>     toloka.client.user_restriction.ProjectUserRestriction(
        >>>         user_id='1ad097faba0eff85a04fe30bc04d53db',
        >>>         private_comment='Low response quality',
        >>>         project_id='5'
        >>>     )
        >>> )
        >>> toloka_client.delete_user_restriction(new_restriction.id)
        ...
    """

    class Scope(toloka.util._extendable_enum.ExtendableStrEnum):
        """A restriction scope.

        Attributes:
            ALL_PROJECTS: Access to all requester's projects is blocked.
            PROJECT: A single project is blocked.
            POOL: A pool is blocked.
            SYSTEM: A system-wide [ban](https://toloka.ai/docs/guide/ban/?form-source=api-ban#ban-platform).
        """

        SYSTEM = 'SYSTEM'
        ALL_PROJECTS = 'ALL_PROJECTS'
        PROJECT = 'PROJECT'
        POOL = 'POOL'

    def __init__(
        self,
        *,
        user_id: typing.Optional[str] = None,
        private_comment: typing.Optional[str] = None,
        will_expire: typing.Optional[datetime.datetime] = None,
        id: typing.Optional[str] = None,
        created: typing.Optional[datetime.datetime] = None
    ) -> None:
        """Method generated by attrs for class UserRestriction.
        """
        ...

    _unexpected: typing.Optional[typing.Dict[str, typing.Any]]
    user_id: typing.Optional[str]
    private_comment: typing.Optional[str]
    will_expire: typing.Optional[datetime.datetime]
    id: typing.Optional[str]
    created: typing.Optional[datetime.datetime]


class AllProjectsUserRestriction(UserRestriction):
    """All projects restriction.

    A Toloker doesn't have access to any of requester's projects.

    Attributes:
        user_id: The ID of the Toloker.
        private_comment: A comment visible to the requester only.
        will_expire: The UTC date and time when the access will be restored by Toloka.
            If the parameter isn't set, then the restriction is active until you remove it calling the `delete_user_restriction` method.
        id: The ID of the restriction.
        created: The UTC date and time when the restriction was applied. Read-only field.
    """

    def __init__(
        self,
        *,
        user_id: typing.Optional[str] = None,
        private_comment: typing.Optional[str] = None,
        will_expire: typing.Optional[datetime.datetime] = None,
        id: typing.Optional[str] = None,
        created: typing.Optional[datetime.datetime] = None
    ) -> None:
        """Method generated by attrs for class AllProjectsUserRestriction.
        """
        ...

    _unexpected: typing.Optional[typing.Dict[str, typing.Any]]
    user_id: typing.Optional[str]
    private_comment: typing.Optional[str]
    will_expire: typing.Optional[datetime.datetime]
    id: typing.Optional[str]
    created: typing.Optional[datetime.datetime]


class PoolUserRestriction(UserRestriction):
    """A pool restriction.

    A Toloker doesn't have access to the pool.

    Attributes:
        user_id: The ID of the Toloker.
        private_comment: A comment visible to the requester only.
        will_expire: The UTC date and time when the access will be restored by Toloka.
            If the parameter isn't set, then the restriction is active until you remove it calling the `delete_user_restriction` method.
        id: The ID of the restriction.
        created: The UTC date and time when the restriction was applied. Read-only field.
        pool_id: The ID of the pool that is blocked.
    """

    def __init__(
        self,
        *,
        user_id: typing.Optional[str] = None,
        private_comment: typing.Optional[str] = None,
        will_expire: typing.Optional[datetime.datetime] = None,
        id: typing.Optional[str] = None,
        created: typing.Optional[datetime.datetime] = None,
        pool_id: typing.Optional[str] = None
    ) -> None:
        """Method generated by attrs for class PoolUserRestriction.
        """
        ...

    _unexpected: typing.Optional[typing.Dict[str, typing.Any]]
    user_id: typing.Optional[str]
    private_comment: typing.Optional[str]
    will_expire: typing.Optional[datetime.datetime]
    id: typing.Optional[str]
    created: typing.Optional[datetime.datetime]
    pool_id: typing.Optional[str]


class ProjectUserRestriction(UserRestriction):
    """A project restriction.

    A Toloker doesn't have access to the project.

    Attributes:
        user_id: The ID of the Toloker.
        private_comment: A comment visible to the requester only.
        will_expire: The UTC date and time when the access will be restored by Toloka.
            If the parameter isn't set, then the restriction is active until you remove it calling the `delete_user_restriction` method.
        id: The ID of the restriction.
        created: The UTC date and time when the restriction was applied. Read-only field.
        project_id: The ID of the project that is blocked.
    """

    def __init__(
        self,
        *,
        user_id: typing.Optional[str] = None,
        private_comment: typing.Optional[str] = None,
        will_expire: typing.Optional[datetime.datetime] = None,
        id: typing.Optional[str] = None,
        created: typing.Optional[datetime.datetime] = None,
        project_id: typing.Optional[str] = None
    ) -> None:
        """Method generated by attrs for class ProjectUserRestriction.
        """
        ...

    _unexpected: typing.Optional[typing.Dict[str, typing.Any]]
    user_id: typing.Optional[str]
    private_comment: typing.Optional[str]
    will_expire: typing.Optional[datetime.datetime]
    id: typing.Optional[str]
    created: typing.Optional[datetime.datetime]
    project_id: typing.Optional[str]


class SystemUserRestriction(UserRestriction):
    """A system-wide restriction.

    Attributes:
        user_id: The ID of the Toloker.
        private_comment: A comment visible to the requester only.
        will_expire: The UTC date and time when the access will be restored by Toloka.
            If the parameter isn't set, then the restriction is active until you remove it calling the `delete_user_restriction` method.
        id: The ID of the restriction.
        created: The UTC date and time when the restriction was applied. Read-only field.
    """

    def __init__(
        self,
        *,
        user_id: typing.Optional[str] = None,
        private_comment: typing.Optional[str] = None,
        will_expire: typing.Optional[datetime.datetime] = None,
        id: typing.Optional[str] = None,
        created: typing.Optional[datetime.datetime] = None
    ) -> None:
        """Method generated by attrs for class SystemUserRestriction.
        """
        ...

    _unexpected: typing.Optional[typing.Dict[str, typing.Any]]
    user_id: typing.Optional[str]
    private_comment: typing.Optional[str]
    will_expire: typing.Optional[datetime.datetime]
    id: typing.Optional[str]
    created: typing.Optional[datetime.datetime]
